@@ -11,27 +11,26 @@ gdalinfo_internal <- function(x, json = TRUE,
                                   wkt_format = "WKT2",
                                   oo = character(),
                                   initial_format = character(),
-                                  options = character()) {
+                                  options = c("-proj4", "-json", "-listmdd")) {
   
   rep_zip <- function(x, y) {
     as.vector(t(cbind(rep(x, length(y)), y)))
   }
   if (length(sd) > 1) message("'sd' argument cannot be vectorized over 'dsn', ignoring all but first value")
   
-  version <- vapour_gdal_version()
-  v3 <- TRUE
-  if (grepl("GDAL 2", version )) v3 <- FALSE
-  extra <- c(if(json) "-json",
-             if (is.numeric(sd) && sd[1L] > 0) c("-sd", sd[1L]),
-             if (stats) "-stats",
-             if (checksum) "-checksum",
-             if (nchar(wkt_format[1]) > 0 && v3) c("-wkt_format", wkt_format[1L]),
-             if (length(oo) > 0 && any(nchar(oo) > 0) ) rep_zip("-oo", oo[nchar(oo) > 0]),
-             if (length(initial_format) > 0 && any(nchar(initial_format) > 0)) rep_zip("-if", initial_format[nchar(initial_format) > 0]))
-  
-  options <- c(options, "-proj4", "-listmdd", extra)
+  #version <- vapour_gdal_version()
+  #v3 <- TRUE
+  # if (grepl("GDAL 2", version )) v3 <- FALSE
+  # extra <- c(if(json) "-json",
+  #            if (is.numeric(sd) && sd[1L] > 0) c("-sd", sd[1L]),
+  #            if (stats) "-stats",
+  #            if (checksum) "-checksum",
+  #            if (nchar(wkt_format[1]) > 0 && v3) c("-wkt_format", wkt_format[1L]),
+  #            if (length(oo) > 0 && any(nchar(oo) > 0) ) rep_zip("-oo", oo[nchar(oo) > 0]),
+  #            if (length(initial_format) > 0 && any(nchar(initial_format) > 0)) rep_zip("-if", initial_format[nchar(initial_format) > 0]))
+  # 
+  #options <- c(options, "-proj4", "-listmdd")
   options <- options[!is.na(options)]  ## cant do unique because repeated arguments possible things like "-if" "GTiff" "-if" "netCDF"
-  
   info <- raster_gdalinfo_app_cpp(x, options)
   info
 }
