@@ -11,5 +11,5 @@ if(!file.exists(testfile)) {
   download.file(sprintf("https://github.com/rwinlib/gdal3/archive/v%s.zip", VERSION), "lib.zip", quiet = TRUE)
   dir.create("../windows", showWarnings = FALSE)
   unzip("lib.zip", exdir = "../windows")
-  unlink("lib.zip")
+  file.remove("lib.zip")
 }
